CREATE TYPE GeoCoord AS (
    Latitude NUMERIC,
    Longitude NUMERIC
);

CREATE TABLE Country (
    Name CHARACTER VARYING(40)
        CONSTRAINT Country_Name_NotNull NOT NULL
        CONSTRAINT Country_Name_Unique UNIQUE,
    Code CHARACTER(2)
        CONSTRAINT Country_Key PRIMARY KEY,
    Capital CHARACTER VARYING(40),
    Province CHARACTER VARYING(40),
    Population NUMERIC
        CONSTRAINT Country_Population_Check CHECK (
             Population >= 0
        ),
    Area NUMERIC
        CONSTRAINT Country_Area_Check CHECK (
             Area >= 0
        )
);

CREATE TABLE Province (
    Name CHARACTER VARYING(40),
    Country CHARACTER(2),
    Population NUMERIC
        CONSTRAINT Province_Population_Check CHECK (
             Population >= 0
        ),
    Area NUMERIC
        CONSTRAINT Province_Area_Check CHECK (
             Area >= 0
        ),
    Capital CHARACTER VARYING(40),
    CapProv CHARACTER VARYING(40),
    CONSTRAINT Province_Key PRIMARY KEY (Country, Name)
);

CREATE TABLE City (
    Name CHARACTER VARYING(40),
    Country CHARACTER(2),
    Province CHARACTER VARYING(40),
    Population NUMERIC
        CONSTRAINT City_Population_Check CHECK (
             Population >= 0
        ),
    Latitude NUMERIC
        CONSTRAINT City_Latitude_Check CHECK (
             (Latitude >= -90) AND (Latitude <= 90)
        ),
    Longitude NUMERIC
        CONSTRAINT City_Longitude_Check CHECK (
             (Longitude >= -180) AND (Longitude <= 180)
        ),
    CONSTRAINT City_Key PRIMARY KEY (Country, Province, Name)
);

CREATE TABLE Continent (
    Name CHARACTER VARYING(20)
        CONSTRAINT Continent_Key PRIMARY KEY,
    Area NUMERIC
        CONSTRAINT Continent_Area_Check CHECK (
             Area >= 0
        )
);

CREATE TABLE encompasses (
    Country CHARACTER(2),
    Continent CHARACTER VARYING(20),
    Percentage NUMERIC
        CONSTRAINT encompasses_Percentage_Check CHECK (
             (Percentage > 0) AND (Percentage <= 100)
        ),
    CONSTRAINT encompasses_Key PRIMARY KEY (Continent, Country)
);

CREATE TABLE borders (
    Country1 CHARACTER(2),
    Country2 CHARACTER(2),
    Length NUMERIC
        CONSTRAINT borders_Length_Check CHECK (
             Length > 0
        ),
    CONSTRAINT borders_Key PRIMARY KEY (Country1, Country2)
);

CREATE TABLE Organization (
    Abbreviation CHARACTER VARYING(15)
        CONSTRAINT Organization_Key PRIMARY KEY,
    Name CHARACTER VARYING(100)
        CONSTRAINT Organization_Name_NotNull NOT NULL
        CONSTRAINT Organization_Name_Unique UNIQUE,
    Established DATE,
    City CHARACTER VARYING(40),
    Province CHARACTER VARYING(40),
    Country CHARACTER(2)
);

CREATE TABLE is_member (
    Organization CHARACTER VARYING(15),
    Country CHARACTER(2),
    Type CHARACTER VARYING(30),
    CONSTRAINT is_member_Key PRIMARY KEY (Country, Organization)
);

CREATE TABLE Economy (
    Country CHARACTER(2)
        CONSTRAINT Economy_Key PRIMARY KEY,
    GDP NUMERIC
        CONSTRAINT Economy_GDP_Check CHECK (
             GDP >= 0
        ),
    Agriculture NUMERIC,
    Industry NUMERIC,
    Services NUMERIC,
    Inflation NUMERIC
);

CREATE TABLE Population (
    Country CHARACTER(2)
        CONSTRAINT Population_Key PRIMARY KEY,
    Population_Growth NUMERIC,
    Infant_Mortality NUMERIC
);

CREATE TABLE Politics (
    Country CHARACTER(2)
        CONSTRAINT Politics_Key PRIMARY KEY,
    Independence DATE,
    Government CHARACTER VARYING(120)
);

CREATE TABLE Language (
    Country CHARACTER(2),
    Name CHARACTER VARYING(50),
    Percentage NUMERIC
        CONSTRAINT Language_Percentage_Check CHECK (
             (Percentage > 0) AND (Percentage <= 100)
        ),
    CONSTRAINT Language_Key PRIMARY KEY (Country, Name)
);

CREATE TABLE Religion (
    Country CHARACTER(2),
    Name CHARACTER VARYING(50),
    Percentage NUMERIC
        CONSTRAINT Religion_Percentage_Check CHECK (
             (Percentage > 0) AND (Percentage <= 100)
        ),
    CONSTRAINT Religion_Key PRIMARY KEY (Country, Name)
);

CREATE TABLE Ethnic_Group (
    Country CHARACTER(2),
    Name CHARACTER VARYING(50),
    Percentage NUMERIC
        CONSTRAINT Ethnic_Group_Percentage_Check CHECK (
             (Percentage > 0) AND (Percentage <= 100)
        ),
    CONSTRAINT Ethnic_Group_Key PRIMARY KEY (Country, Name)
);

CREATE TABLE located (
    City CHARACTER VARYING(40)
        CONSTRAINT located_City_NotNull NOT NULL,
    Province CHARACTER VARYING(40)
        CONSTRAINT located_Province_NotNull NOT NULL,
    Country CHARACTER(2)
        CONSTRAINT located_Country_NotNull NOT NULL,
    River CHARACTER VARYING(30),
    Lake CHARACTER VARYING(30),
    Sea CHARACTER VARYING(30)
);

CREATE TABLE River (
    Name CHARACTER VARYING(30)
        CONSTRAINT River_Key PRIMARY KEY,
    River CHARACTER VARYING(30),
    Lake CHARACTER VARYING(30),
    Sea CHARACTER VARYING(30),
    Length NUMERIC
        CONSTRAINT River_Length_Check CHECK (
             Length >= 0
        )
);

CREATE TABLE Mountain (
    Name CHARACTER VARYING(30)
        CONSTRAINT Mountain_Key PRIMARY KEY,
    Height NUMERIC
        CONSTRAINT Mountain_Height_Check CHECK (
             Height >= 0
        ),
    Coordinates GeoCoord
        CONSTRAINT Mountain_Coordinates_Check CHECK (
             ((Coordinates).Longitude >= -180) AND
             ((Coordinates).Longitude <= 180) AND
             ((Coordinates).Latitude >= -90) AND
             ((Coordinates).Latitude <= 90)
        )
);

CREATE TABLE Lake (
    Name CHARACTER VARYING(30)
        CONSTRAINT Lake_Key PRIMARY KEY,
    Area NUMERIC
        CONSTRAINT Lake_Area_Check CHECK (
             Area >= 0
        )
);

CREATE TABLE Sea (
    Name CHARACTER VARYING(30)
        CONSTRAINT Sea_Key PRIMARY KEY,
    Depth NUMERIC
        CONSTRAINT Sea_Depth_Check CHECK (
             Depth >= 0
        )
);

CREATE TABLE Island (
    Name CHARACTER VARYING(30)
        CONSTRAINT Island_Key PRIMARY KEY,
    Islands CHARACTER VARYING(30),
    Area NUMERIC
        CONSTRAINT Island_Area_Check CHECK (
             Area >= 0
        ),
    Coordinates GeoCoord
        CONSTRAINT Island_Coordinates_Check CHECK (
             ((Coordinates).Longitude >= -180) AND
             ((Coordinates).Longitude <= 180) AND
             ((Coordinates).Latitude >= -90) AND
             ((Coordinates).Latitude <= 90)
        )
);

CREATE TABLE Desert (
    Name CHARACTER VARYING(30)
        CONSTRAINT Desert_Key PRIMARY KEY,
    Area NUMERIC
        CONSTRAINT Desert_Area_Check CHECK (
             Area >= 0
        )
);

CREATE TABLE geo_river (
    River CHARACTER VARYING(30),
    Country CHARACTER(2),
    Province CHARACTER VARYING(40),
    CONSTRAINT geo_river_Key PRIMARY KEY (Country, Province, River)
);

CREATE TABLE geo_mountain (
    Mountain CHARACTER VARYING(30),
    Country CHARACTER(2),
    Province CHARACTER VARYING(40),
    CONSTRAINT geo_mountain_Key PRIMARY KEY (Country, Province, Mountain)
);

CREATE TABLE geo_lake (
    Lake CHARACTER VARYING(30),
    Country CHARACTER(2),
    Province CHARACTER VARYING(40),
    CONSTRAINT geo_lake_Key PRIMARY KEY (Country, Province, Lake)
);

CREATE TABLE geo_sea (
    Sea CHARACTER VARYING(30),
    Country CHARACTER(2),
    Province CHARACTER VARYING(40),
    CONSTRAINT geo_sea_Key PRIMARY KEY (Country, Province, Sea)
);

CREATE TABLE geo_island (
    Island CHARACTER VARYING(30),
    Country CHARACTER(2),
    Province CHARACTER VARYING(40),
    CONSTRAINT geo_island_Key PRIMARY KEY (Country, Province, Island)
);

CREATE TABLE geo_desert (
    Desert CHARACTER VARYING(30),
    Country CHARACTER(2),
    Province CHARACTER VARYING(40),
    CONSTRAINT geo_desert_Key PRIMARY KEY (Country, Province, Desert)
);

CREATE TABLE merges_with (
    Sea1 CHARACTER VARYING(30),
    Sea2 CHARACTER VARYING(30),
    CONSTRAINT merges_with_Key PRIMARY KEY (Sea1, Sea2)
);
